@@ -7,12 +7,13 @@ use AE;
 use AnyEvent;
 use AnyEvent::Handle;
 use AnyEvent::Socket qw( tcp_connect );
+use Protocol::WebSocket::Request;
 use Protocol::WebSocket::Handshake::Client;
 use AnyEvent::WebSocket::Connection;
 use PerlX::Maybe qw( maybe provided );
 
 # ABSTRACT: WebSocket client for AnyEvent
-our $VERSION = '0.35'; # VERSION
+our $VERSION = '0.36'; # VERSION
 
 
 has timeout => (
@@ -31,6 +32,37 @@ has ssl_ca_file => (
 );
 
 
+has protocol_version => (
+  is => 'ro',
+);
+
+
+has subprotocol => (
+  is     => 'ro',
+  coerce => sub { ref $_[0] ? $_[0] : [$_[0]] },
+);
+
+
+has http_headers => (
+  is => 'ro',
+  coerce => sub {
+    ref $_[0] eq 'ARRAY' ? $_[0] : do {
+      my $h = shift;
+      [
+        map {
+          my($k,$v) = ($_, $h->{$_});
+          $v = [$v] unless ref $v;
+          map { $k => $_ } @$v;
+          # sorted to make testing easier.
+          # may be removed in the future
+          # so do not depend on it.
+        } sort keys %$h
+      ],
+    };
+  },
+);
+
+
 sub connect
 {
   my($self, $uri) = @_;
@@ -57,10 +89,20 @@ sub connect
       $done->croak("unable to connect");
       return;
     }
+    my $req = Protocol::WebSocket::Request->new( maybe headers => $self->http_headers );
     my $handshake = Protocol::WebSocket::Handshake::Client->new(
-      url => $uri->as_string,
+            url     => $uri->as_string,
+      maybe version => $self->protocol_version,
+            req     => $req,
     );
     
+    my %subprotocol;
+    if($self->subprotocol)
+    {
+      %subprotocol = map { $_ => 1 } @{ $self->subprotocol };
+      $handshake->req->subprotocol(join(',', @{ $self->subprotocol }));
+    }
+    
     my $hdl = AnyEvent::Handle->new(
                                                       fh       => $fh,
       provided $uri->secure,                          tls      => 'connect',
@@ -78,7 +120,7 @@ sub connect
                                                         { warn $msg }
                                                       },
     );
-    
+
     $hdl->push_write($handshake->to_string);
     $hdl->on_read(sub {
       $handshake->parse($_[0]{rbuf});
@@ -91,8 +133,30 @@ sub connect
       }
       elsif($handshake->is_done)
       {
+        my $sb;
+        if($self->subprotocol)
+        {
+          $sb = $handshake->res->subprotocol;
+          if(defined $sb)
+          {
+            unless($subprotocol{$sb})
+            {
+              $done->croak("subprotocol mismatch, requested: @{[ join ', ', @{ $self->subprotocol } ]}, got: $sb");
+            }
+          }
+          else
+          {
+            $done->croak("no subprotocol in response");
+          }
+        }
         undef $handshake;
-        $done->send(AnyEvent::WebSocket::Connection->new(handle => $hdl, masked => 1));
+        $done->send(
+          AnyEvent::WebSocket::Connection->new(
+                  handle      => $hdl,
+                  masked      => 1,
+            maybe subprotocol => $sb,
+          )
+        );
         undef $hdl;
         undef $done;
       }
@@ -115,7 +179,7 @@ AnyEvent::WebSocket::Client - WebSocket client for AnyEvent
 
 =head1 VERSION
 
-version 0.35
+version 0.36
 
 =head1 SYNOPSIS
 
@@ -195,6 +259,44 @@ not be verified.  The default is false.
 Provide your own CA certificates file instead of using the system default for
 SSL/TLS verification.
 
+=head2 protocol_version
+
+The protocol version.  See L<Protocol::WebSocket> for the list of supported
+WebSocket protocol versions.
+
+=head2 subprotocol
+
+List of subprotocols to request from the server.  This class will throw an
+exception if none of the protocols are supported by the server.
+
+=head2 http_headers
+
+Extra headers to include in the initial request.  May be either specified
+as a hash reference, or an array reference.  For example:
+
+ AnyEvent::WebSocket::Client->new(
+   http_headers => {
+     'X-Foo' => 'bar',
+     'X-Baz' => [ 'abc', 'def' ],
+   },
+ );
+ 
+ AnyEvent::WebSocket::Client->new(
+   http_headers => [
+     'X-Foo' => 'bar',
+     'X-Baz' => 'abc',
+     'X-Baz' => 'def',
+   ],
+ );
+
+Will generate:
+
+ X-Foo: bar
+ X-Baz: abc
+ X-Baz: def
+
+Although, the order cannot be guaranteed when using the hash style.
+
 =head1 METHODS
 
 =head2 connect
